<?php

namespace Drupal\discourse_comments_plus\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Class DiscourseCommentsSettingsForm.
 */
class DiscourseCommentsSettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The discourse API client.
   *
   * @var \Drupal\discourse_comments_plus\DiscourseApiClient
   */
  protected $discourseApiClient;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The token service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->discourseApiClient = $container->get('discourse_comments_plus.discourse_api_client');
    $instance->moduleHandler = $container->get('module_handler');
    $instance->token = $container->get('token');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'discourse_comments_plus.discourse_comments_settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'discourse_comments_plus_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('discourse_comments_plus.discourse_comments_settings');
    $form['base_url_of_discourse'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Discourse Base URL'),
      '#description' => $this->t('Please enter url without trailing / character. Example: https://test.trydiscourse.com'),
      '#maxlength' => 256,
      '#size' => 64,
      '#default_value' => $config->get('base_url_of_discourse'),
      '#required' => TRUE,
    ];

    $form['checkbox_internal_base_url_of_discourse'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Internal Discourse Base URL is different'),
      '#default_value' => $config->get('checkbox_internal_base_url_of_discourse'),
    ];

    $form['internal_base_url_of_discourse'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Discourse Internal Base URL'),
      '#description' => $this->t('Please enter url without trailing / character. Example: http://discourse:3000'),
      '#maxlength' => 256,
      '#size' => 64,
      '#default_value' => $config->get('internal_base_url_of_discourse'),
      '#states' => [
        'visible' => [
          ':input[name="checkbox_internal_base_url_of_discourse"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="checkbox_internal_base_url_of_discourse"]' => ['checked' => TRUE],
        ],
      ]
    ];

    $form['sso_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Discourse SSO Secret'),
      '#description' => $this->t('Make sure to use the same secret in Discourse and Drupal. To configure SSO, go to <br><code>/admin/site_settings/category/login?filter=discourse%20connect%20provider</code>'),
      '#maxlength' => 256,
      '#size' => 64,
      '#required' => TRUE,
      '#default_value' => $config->get('sso_secret'),
    ];

    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Discourse Api Key'),
      '#maxlength' => 256,
      '#size' => 64,
      '#required' => TRUE,
      '#default_value' => $config->get('api_key'),
    ];

    $form['api_user_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Discourse Api Username'),
      '#maxlength' => 256,
      '#size' => 64,
      '#required' => TRUE,
      '#default_value' => $config->get('api_user_name'),
    ];

    $form['cache_lifetime'] = [
      '#type' => 'number',
      '#title' => $this->t('Cache Lifetime (in minutes)'),
      '#size' => 64,
      '#required' => TRUE,
      // Default cache lifetime 60 minutes.
      '#default_value' => $config->get('cache_lifetime') ? $config->get('cache_lifetime') : 60,
    ];

    $form['footer_template'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Footer Template'),
      '#format' => 'full_html',
      '#rows' => 2,
      '#required' => FALSE,
      '#default_value' => $config->get('footer_template'),
      '#description' => $this->t('Available tokens: [node:title], [node:url], [node:author], etc.'),
    ];

    if ($this->moduleHandler->moduleExists('token')) {
      $form['footer_template']['#description'] .= ' ' . $this->t('Click the "Browse available tokens" button to see all available tokens.');
      $form['footer_template']['token_tree'] = [
        '#theme' => 'token_tree_link',
        '#token_types' => ['node'],
        '#show_restricted' => FALSE,
        '#global_types' => FALSE,
        '#weight' => 90,
      ];
    }

    if (isset($config) && !empty($config)) {
      $base_url = $config->get('base_url_of_discourse');
      if (isset($base_url) && $base_url != '') {
        $categories = $this->discourseApiClient->getCategories();
        if ($categories) {
          $options = [];
          foreach ($categories['category_list']['categories'] as $cat) {
            $options[$cat['id']] = $cat['name'];
          }
          $form['default_category'] = [
            '#type' => 'select',
            '#title' => $this->t('Default category to post to'),
            '#options' => $options,
            '#default_value' => $config->get('default_category'),
          ];
        }
      }
    }

    // Prepare options for content type checkboxes.
    $content_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    $options = [];
    foreach ($content_types as $key => $content_type) {
      $options[$key] = $content_type->get('name');
    }

    $form['content_types_enabled_for_discourse'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Content types for which Discourse Comments Integration should be enabled by default'),
      '#options' => $options,
      '#default_value' => $config->get('content_types_enabled_for_discourse') == NULL ? [] : $config->get('content_types_enabled_for_discourse'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $content_types_enabled = $form_state->getValue('content_types_enabled_for_discourse');
    $footer_template = $form_state->getValue('footer_template');
    $this->config('discourse_comments_plus.discourse_comments_settings')
      ->set('base_url_of_discourse', $form_state->getValue('base_url_of_discourse'))
      ->set('checkbox_internal_base_url_of_discourse', $form_state->getValue('checkbox_internal_base_url_of_discourse'))
      ->set('internal_base_url_of_discourse', $form_state->getValue('internal_base_url_of_discourse'))
      ->set('sso_secret', $form_state->getValue('sso_secret'))
      ->set('api_key', $form_state->getValue('api_key'))
      ->set('api_user_name', $form_state->getValue('api_user_name'))
      ->set('cache_lifetime', $form_state->getValue('cache_lifetime'))
      ->set('default_category', $form_state->getValue('default_category'))
      ->set('content_types_enabled_for_discourse', $content_types_enabled)
      ->set('footer_template', $footer_template['value'])
      ->save();
  }

}
